#include "ThunderDrive.h"
#include "WPILib.h"
#include "Kicker.h"
#include "ThunderCam.h"
#include "NewHanger.h"
#include "ThunderGlow.h"
#ifndef CONTROLS_H
#define CONTROLS_H
//This file is for the operator control functions.
//Most (if not all) controls functions will go here.




class Controls{

private:
    Joystick *leftStick;
    Joystick *rightStick;
    ThunderDrive *drive;
    Kicker *kicker;
    DriverStation *driverStation;
    DriverStationEnhancedIO & driverStationio;
    DriverStationLCD * driverStationLCD;
    ThunderCam *tCamera;
    NewHanger *tHanger;
    ThunderGlow *tGlow;
    bool hasEnhancedIO;
    double nextDebugTime;
    double nextEIOCheckTime;
    
    
    
    void configureDriversStation(void);
    
    //Takes the joystick, and gets the buttons, 
    //Y-value, and returns the correct value to drive 
    //the motors at the correct power.
    float computePower(Joystick *stick);  
    
    
    
    
public:
    /* Checks for presence of Enhanced IO, enables its use if found, and
     * configures each digital pin as input or output once found.
     * NOTE: Call checkDisables()/runControls() to re-read all controls
     * after calling this to process new values based on the new state of
     * enh io presence
     */
    void checkForEnhancedIO(void);
    
	void checkDisables(); //Called in autonomous and disabled - checks the broken switches
	
    void runControls();    //Called by thunderBot.  Executed every loop.  Runs all internal code to drive motors/Get buttons/ Et Cetera

    float getAutoDelay();
    
    bool isKickerWorking();
    
    //the debugging function to print each robot part (ex. kicker, thunderdriver) in the user messages part of the driver station
    void runDebugging();
    
    //Updates all the Driverstation Feedback 
    //(Lights, bargraphs, Etc.)
    void driverstationFeedback();  

    // clears everything in the user messages box
    void clearMessages();
    Controls(Joystick *stickLeft, Joystick *stickRight, ThunderDrive *drive, Kicker *kicker, ThunderCam *camera, NewHanger *hanger);
    ~Controls();
    
    
	
};

#endif
