#include "WPILib.h"
#include "EncoderPID.h"
#include "Math.h"
#include "IOMAP.h"
#include "Kicker.h"
#include "timers.h"
#include "CANJaguar/CANJaguar.h"


	bool Kicker::FloatEquality(float test, float comparison, float tolerance) {
		if((test >= comparison) && (test <= comparison + tolerance)){
			return true;
		}
			return false;
	}
	Kicker::Kicker() {
		KickerMotor1 = new Relay(RELAY_KICKER_MOTOR_LEFT);
		KickerMotor2 = new Relay(RELAY_KICKER_MOTOR_RIGHT);
		EMagnet = new Relay(RELAY_ELECTRO_MAGNET, Relay::kForwardOnly);
		EMagnet2 = new Relay(RELAY_ELECTRO_MAGNET_2, Relay::kForwardOnly);
		
		BeaterBrush = new CANJaguar(BEATER_BRUSH);
		
		MagnetTouched = new DigitalInput(ELECTROMAGNET_CONTACT);
		MagPot = new AnalogChannel(MAG_POT);
		BannerSensor = new DigitalInput(BALL_BANNER_SENSOR);
		
		KickerFullForward = new DigitalInput(KICKER_MAX_FORWARD);
		KickerFullReverse = new DigitalInput(KICKER_MAX_REVERSE);
		
		relayPulse = 0;
		isTimeoutRunning = false;
		emagnetSwitchBroken = false;
		ballSensorBroken = false;
		powerLevelSensorBroken = false;
		desiredPowerLevel = 0;
		kickerPosition = 0;
		LastSwitchState = 0;
		//Pingas = 9001;
		preMatchTime = 0;
		fireTime = 0;
		timeoutTime = INT_MAX;
		retractedTime = 0;
		LastPotValue = 0;

		kickerPreMatch = false;
		BeaterBrushDirection = 0;
	}
	Kicker::~Kicker() {
	    delete KickerMotor1;
	    delete KickerMotor2;
	    delete BeaterBrush;
	    delete EMagnet;
	    delete EMagnet2;
	    delete MagnetTouched;
	    delete MagPot;
	    delete BannerSensor;
	    delete KickerFullForward;
	    delete KickerFullReverse;
	    //invoke Watson("OLOLOL");
	}
	bool Kicker::Fire() {
		if(CheckPowerLevel() && ShooterReady()){
			if(BallAvailable()){
				EMagnet->Set(Relay::kOff);
				EMagnet2->Set(Relay::kOff);
				Reset();
				fireTime = GetTime() + Fire_Delay;
				BeaterBrush->Set(0.0f);
				return true;
			}
		}
		return false;
	}
	void Kicker::SetPowerDesired(float power) {
		if(kickerPreMatch){
			if (powerLevelSensorBroken) {
				power = 0.5;
			} else {
				power = 1;
			}
		}
		desiredPowerLevel = power;
	}
	void Kicker::ProcessKicker(){
		relayPulse++;
		if(relayPulse < Pulse_Rate){
			KickerMotor1->Set(Relay::kOff);
			KickerMotor2->Set(Relay::kOff);
			return;
		}
		relayPulse = 0;
		
		/*
		 *   Pre-Match Check code starts here
		 */
		
		if(kickerPreMatch && ((GetTime() > preMatchTime) || CheckPowerLevel())){
			printf("Pre-match complete\n");
			kickerPreMatch = false;
		}


		// If not in the middle of a fire, run beater brush as commanded
		// (Turned off when fired, back on (if so desired) here)
		if(GetTime() >= fireTime){
			BeaterBrush->Set(BeaterBrushDirection * Beater_Speed);
			//BeaterBrush->Set(BeaterBrushDirection * beaterSpeed);
		}
		
		
		/*
		 *   Driver Override code starts here
		 */		
		if(powerLevelSensorBroken){

			if(GetTime() >= fireTime){
				EMagnet->Set(Relay::kOn);
				EMagnet2->Set(Relay::kOn);
			}
			
			if((desiredPowerLevel >= .75) && (KickerFullForward->Get() == 1)){
				KickerMotor1->Set(Relay::kForward);
				KickerMotor2->Set(Relay::kForward);
			}
			else if((desiredPowerLevel <= .25)  && (KickerFullReverse->Get() == 1)){
				KickerMotor1->Set(Relay::kReverse);
				KickerMotor2->Set(Relay::kReverse);
			} else {
				// Either input is in center (turn off on purpose) or
				// at limit
				KickerMotor1->Set(Relay::kOff);
				KickerMotor2->Set(Relay::kOff);
			}					
		}
		
		/*
		 *   Normal operation code starts here
		 */
		
		else{
			//Start 2 second timer
			if(LastPotValue < 0 && GetPowerLevel() >= 0){
				retractedTime = GetTime() + 2;
			}
			
			//Broken E-magnet switch
			if((MagnetTouched->Get() == 1 && KickerFullForward->Get() == 0) || (GetTime() >= timeoutTime && isTimeoutRunning)){
				emagnetSwitchBroken = true;
				//printf("MAGNET SENSOR BROKEN!!!\n");
			}
			
			//Go forward if the magnet isn't touched
			if((MagnetTouched->Get() == 1) && (!emagnetSwitchBroken) && (KickerFullForward->Get() == 1)){
				KickerMotor1->Set(Relay::kForward);
				KickerMotor2->Set(Relay::kForward);
				//printf("Magnet not touched\n");
				if(!isTimeoutRunning){
					timeoutTime = GetTime() + 5; 
					isTimeoutRunning = true;
				}
			}
			else{
				float magvolt = GetPowerLevel();
				isTimeoutRunning = false;
				//Emagnet must be touched, so turn it on
				if(GetTime() >= fireTime){
					EMagnet->Set(Relay::kOn);
					EMagnet2->Set(Relay::kOn);
				}
				
				//might not want to look at kicker limit switch here?
				//Commented out - with new limit on getpowerlevel
				//we shouldn't need this
				
				/*
				if(magvolt < 0 && KickerFullReverse->Get() == 1){
					KickerMotor1->Set(Relay::kReverse);
					KickerMotor2->Set(Relay::kReverse);
					//printf("Too far forward\n");
				}
				
				
				else{
				*/
				if(CheckPowerLevel()){
					KickerMotor1->Set(Relay::kOff);
					KickerMotor2->Set(Relay::kOff);
				}
				else{	
					// This pulls the kicker backwards if:
						// Not as powerful as desired and the backmost limit switch is not pushed
						// The kicker is forward of the frame perimeter
					if((magvolt <= desiredPowerLevel) && (KickerFullReverse->Get() == 1)){
						KickerMotor1->Set(Relay::kReverse);
						KickerMotor2->Set(Relay::kReverse);
						//printf("Need more power\n");
					}
					else{
						// More powerful than desired
						if((magvolt > desiredPowerLevel) && (KickerFullForward->Get() == 1)){
							KickerMotor1->Set(Relay::kForward);
							KickerMotor2->Set(Relay::kForward);
							//printf("Too much power\n");
						}
						else{
							KickerMotor1->Set(Relay::kOff);
							KickerMotor2->Set(Relay::kOff);
							//printf("MagPot=%f Desir=%f KFF=%i KFR=%i\n", (double)magvolt, (double)desiredPowerLevel, KickerFullForward->Get(), KickerFullReverse->Get());
						}
					}
				}
				//}	
			}		
		}
		LastSwitchState = MagnetTouched->Get();
		LastPotValue = GetPowerLevel();
	}

	void Kicker::TurnBeaterBrush(int direction) {
		BeaterBrushDirection = direction;
	}

	bool Kicker::CheckPowerLevel() {
		if(powerLevelSensorBroken){
			return true;
		}
		if(KickerFullForward->Get() == 0 && desiredPowerLevel > GetPowerLevel()){
			return true;
		}
		return FloatEquality(GetPowerLevel(), GetPowerDesired(), Power_Level_Tolerance);
	}
	float Kicker::GetPowerLevel() {
		if(powerLevelSensorBroken){
			return GetPowerDesired();
		}
		return GetMagPot();
	}
	
	float Kicker::GetRawPot() {
		return MagPot->GetVoltage();
	}
	
	float Kicker::GetMagPot() {
		//printf("MagValue %d \n", MagPot->GetValue());
		//return MagPot->GetVoltage();
		float MPnormalized = MagPot->GetVoltage() - Kicker_Frame_Perimeter;
		return (MPnormalized) / (Kicker_Fully_Charged - Kicker_Frame_Perimeter);
	}
	
	float Kicker::GetPowerDesired() {
		return desiredPowerLevel;
	}

	bool Kicker::ShooterReady() {
		if(GetPowerLevel() < 0){
			return false;
		}
		if(GetTime() > retractedTime){
			return true;
		}
		return false;
	}
	
	bool Kicker::BallAvailable() {
		if(ballSensorBroken){
			return true;
		}
		//The one sensor that is pulled low vvvvvv
		return BannerSensor->Get();
	}
	
	
	int Kicker::GetForwardLimit(){
		return KickerFullForward->Get();
	}
	
	int Kicker::GetBackwardLimit(){
		return KickerFullReverse->Get();
	}
	
	int Kicker::GetMagnetTouched(){
		return MagnetTouched->Get();
	}
	
	int Kicker::GetRawBallSensor() {
		return BannerSensor->Get();
	}
	
	bool Kicker::GetBallSensorBroken() {  
		return ballSensorBroken;
	}
	
	bool Kicker::GetPowerLevelSensorBroken() { 
		return powerLevelSensorBroken;
	}
	
	void Kicker::SetBallSensorBroken(bool value) {
		ballSensorBroken = value;
	}
	
	void Kicker::SetPowerLevelSensorBroken(bool value) {
		powerLevelSensorBroken = value;
	}
	void Kicker::SetPreMatch() {
		if (!kickerPreMatch)
			printf("Pre-match begins\n");

		kickerPreMatch = true;
		preMatchTime = GetTime() + Prematch_Delay;
		if (powerLevelSensorBroken) {
			desiredPowerLevel = 0.5;
		} else {
			desiredPowerLevel = 1;
		}
	}
	bool Kicker::GetPreMatchDone() {
		return !kickerPreMatch;
	}
	void Kicker::Reset() {
		emagnetSwitchBroken = false;
		timeoutTime = INT_MAX;
	}
