#include "WPILib.h"
#include "EncoderPID.h"
#include "Math.h"
#include "IOMAP.h"
#include "NewHanger.h"
#include "CANJaguar/CANJaguar.h"

//P, I, and D components
#define PC 0.17 //1.2
#define IC 0.01
#define DC 0.0

//The setpoint multiple the PID loop strives for
#define SetpointMultiple 9 //1.5 Before ramping EQ. update

//How much faster the hanger extends vs retracts
#define RaiseMultiple 9001

//Min and Max power the RAMPING CODE will put the SETPOINT to go to
//These go between 1 and 0
#define MinPower .85 //.55 Before ramping EQ. update
#define MaxPower 1.4 

//THESE ARE THE ONES THAT WORKED a long time ago DO NOT LOSE
//#define MinPower .15
//#define MaxPower 1

//The number of seconds that goes by between samples for the PID
#define Sample_Period .2

//Pi. Duh. ;)
#define Pi 3.141592659

// A safe default for the manual hanger power; also used for hanger down
// when hanger lock is not engaged.
#define SafeManualHangerPower (0.3f)

	NewHanger::NewHanger() {
		Raiser = new CANJaguar(HANGER_WINCH);
		Lock = new Servo(HANGER_LOCK);
		Lower_Limit = new DigitalInput(HANGER_LIMIT);
		
		Potentiometer = new AnalogDifference(HANGER_POT);
		HangerPID = new PIDController(PC, IC, DC, Potentiometer, Raiser, Sample_Period);
		
		HangerBroken = false;
		LockEnabled = false;
		
		LastDirection = 0;
		
		/*
		 * HANGER UP/DOWN CONSTANTS HERE
		 * This sets the POT values that represent "full up" and "full down"
		 */
		
		HangerDownSetpoint = 3.78; //3.56 old setpoint
		HangerUpSetpoint = 1.42; //1.65;
		
		ManualHangerPower = SafeManualHangerPower;
	}
	NewHanger::~NewHanger() {
	    delete HangerPID;
	    delete Potentiometer;
	    delete Lock;
	    delete Raiser;
	    delete Lower_Limit;
		//invoke DrOctagonapus("Bwaaa");
	}
	
	
	void NewHanger::SetManualHangerPower(float power) {
	    if (power < 0.0f)
	        power = 0.0f;
	    if (power > 1.0f)
	        power = 1.0f;
	    ManualHangerPower = power;
	}
	
	void NewHanger::DebugHanger(){
		printf("Pot: %f , UD: %f , Srv %f\n "
			   "PID %f , Speed %f , LastPOT %f \n", 
			   Potentiometer->GetVoltage(), 
			   Raiser->Get(), 
			   Lock->Get(),
			   HangerPID->Get(),  
			   Potentiometer->getLastSpeed(),
			   Potentiometer->getLastReading());
	}
	
	float NewHanger::DebugPot(){
		return Potentiometer->GetVoltage();
	}
	float NewHanger::DebugRaiser(){
		return Raiser->Get();
	}
	float NewHanger::DebugPID(){
		return HangerPID->Get();
	}
	float NewHanger::DebugLock(){
		return Lock->Get();
	}
	double NewHanger::DebugSpeed(){
		return Potentiometer->getLastSpeed();
	}
	float NewHanger::DebugLimit(){
		return Lower_Limit->Get();
	}
	float NewHanger::DebugDesiredSpeed(){
		return CalcSetpoint;
	}
	
	void NewHanger::ProcessHanger(){
		CalculateRamp();
		if(!HangerBroken && (((Potentiometer->GetVoltage() >= HangerDownSetpoint) && (LastDirection == -1)) || ((Potentiometer->GetVoltage() <= HangerUpSetpoint) && (LastDirection == 1))) || ((Lower_Limit->Get() == 0) && (LastDirection == -1))){
			HangerPID->Reset();
			Raiser->Set(0);
			//printf("Pot voltage: %f, HangerDownSetpoint: %f, HangerBroken: %d, Down Limit: %d\n", Potentiometer->GetVoltage(), HangerDownSetpoint, (int)HangerBroken);			//3.23 at the bottom
		}
		else{
			if(LastDirection == -1){
				if(!LockEnabled){
					HangerPID->SetSetpoint(-(CalcSetpoint / 2));
				}
				else{
					HangerPID->SetSetpoint(-CalcSetpoint);
				}
			}
			else{
				if(LastDirection == 1){
					HangerPID->SetSetpoint(RaiseMultiple * CalcSetpoint);
				}
			}
		}
	}
	
	//Moves the hange motor up, down, or not at all
	//Pass a 1 for up, a 0 for stop, and a -1 for down
	//Automatically stops the hanger if it has reached it's extent.
	void NewHanger::HangerUpDown(int direction){
		LastDirection = direction;
		if(HangerBroken){
			if((direction == 1) && (!LockEnabled)/* && (Potentiometer->GetVoltage() >= HangerUpSetpoint)*/){
				Raiser->Set(ManualHangerPower);
			}
			else{
				if((direction == -1)  /*&& (Potentiometer->GetVoltage() <= HangerDownSetpoint)*/){
				    if (LockEnabled)
				        Raiser->Set(-ManualHangerPower);
				    else
				        Raiser->Set(-SafeManualHangerPower);
				}
				else{
					Raiser->Set(0);
				}
			}
		}
		else{
			if((direction == 1) && (!LockEnabled) && (Potentiometer->GetVoltage() >= HangerUpSetpoint)){
				HangerPID->SetSetpoint(RaiseMultiple * CalcSetpoint);
				HangerPID->SetOutputRange(0.0 , 1.0);
				HangerPID->Enable();
			}
			else{
				if((direction == -1)  && (Potentiometer->GetVoltage() <= HangerDownSetpoint)){
					if(!LockEnabled){
						HangerPID->SetSetpoint(-(CalcSetpoint / 2));
					}
					else{
						HangerPID->SetSetpoint(-CalcSetpoint);
					}
					HangerPID->SetOutputRange(-1 /*.75  Before ramping EQ. update */, 0.0);
					HangerPID->Enable();
				}
				else{
					HangerPID->Disable();
				}
			}
		}
	}
	
	//Shouldn't be called now, but just in case
	void NewHanger::HangerLeftRight(int direction){
		return;
	}
	
	void NewHanger::HangerLock(bool lock){
		if(lock){
			Lock->Set(ServoExtent);
			LockEnabled = true;
		}
		else{
			Lock->Set(ServoMin);
			LockEnabled = false;
		}
	}

	void NewHanger::SetHangerSensorBroken(bool value){
		HangerBroken = value;
	}
	
	bool NewHanger::IsBroken(){
		return HangerBroken;
	}
	void NewHanger::Reset(){
		HangerPID->Reset();
		HangerPID->Disable();
		Raiser->Set(0);
	}


	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	void NewHanger::CalculateRamp(){
		//Normalizes the pot value
		double CalcPotValue = (Potentiometer->GetVoltage() - HangerDownSetpoint)/(HangerUpSetpoint - HangerDownSetpoint);
		//Calculates the number to multiply the ramping power by to
		//get it within the range it should be
		double q = 1-(MinPower+(1-MaxPower));
		//Does some calculations :/
		//double m = fabs(CalcPotValue - .5);
		//Need absolute Value here
		//		CalcSetpoint = SetpointMultiple * (q * (.5 * sin(2 * Pi * (CalcPotValue - .25))) +
		//									   MinPower + (q / 2)); Before ramping EQ. update
		CalcSetpoint = SetpointMultiple * (q * (.5 * sin(2 * Pi * (CalcPotValue - .25)))) +
										   MinPower + (SetpointMultiple * q / 2);
	}