#include "WPILib.h"
#include "CANJaguar/CANJaguar.h"
#ifndef HANGER_H
#define HANGER_H
#define LeftRightSpeed 0.6 //Sets the speed at which the hanger will move sideways
						   //Make sure this is a FLOAT between 0 and 1
#define ServoExtent 0.57//Sets how far the servo moves when it is set in the "locked" position
#define ServoMin .22//Sets what the minimum servo value is


class AnalogDifference : public AnalogChannel {
private:
	double LastReading;
	double lastpid;
public:
	AnalogDifference(UINT32 channel) : AnalogChannel(channel){
		LastReading = AnalogChannel::PIDGet();
	}
	double PIDGet(){
		// ADC Values!!! 10 bit
		double tempGet = AnalogChannel::PIDGet();
		double output = tempGet - LastReading;
		LastReading = tempGet;
		lastpid = output;
		return output;
	}

	double getLastSpeed(){
		return lastpid;
	}
	double getLastReading(){
		return LastReading;
	}

};





class NewHanger {
public:
	NewHanger();
	~NewHanger();
	
	//Stops the hanger when it is at it's extent
	//Note: THIS MUST BE CALLED REPEATEDLY
	//You can NOT call this once and leave it. 
	//If you do not call this repeatedly, VERY VERY VERY bad things will happen
	//Not joking.
	void ProcessHanger();
	
	//Moves the hanger motor up, down, or not at all
	//Pass a 1 for up, a 0 for stop, and a -1 for down
	//Automatically stops the hanger if it has reached it's extent.
	//Will not try to extend the arm if the lock is enabled
	void HangerUpDown(int direction);
	
	//Moves the hanger left, right, or not at all
	//Pass a -1 for left, a 0 for stop, and a 1 for right
	void HangerLeftRight(int direction);
	
	//Locks the hanger arm so that it can only retract.
	//Pass a true for lock, and a false for unlcok
	void HangerLock(bool lock);

	//Tells me if the hanger sensors are broken
	//If this is true, manual control (no PID used) will be instated
	//In case you couldn't guess: true = it's broken, false = it's not
	void SetHangerSensorBroken(bool value);
	
	//Sets the power level used to run the hanger when set to "broken" mode
	// and the hanger is running manually (without speed control)
	void SetManualHangerPower(float power);
	
	//Prints out values
	void DebugHanger();

	float DebugPot();
	float DebugRaiser();
	float DebugPID();
	float DebugLock();
	double DebugSpeed();
	float DebugLimit();
	float DebugDesiredSpeed();
	bool IsBroken();
	void Reset();
	
	
	//Uses magical equations to calculate a ramp for the setpoint to be
	void CalculateRamp();
private:
	PIDController * HangerPID;
	bool LockEnabled;
	bool HangerBroken;
	int LastDirection;
	
	double HangerDownSetpoint;
	double HangerUpSetpoint;
	
	AnalogDifference * Potentiometer;
	DigitalInput * Lower_Limit;
	CANJaguar * Raiser;
//	CANJaguar * Swinger;
	Servo * Lock;
	
	float CalcSetpoint;
	float ManualHangerPower;
};
#endif

