/*----------------------------------------------------------------------------*/
/* Copyright (c) FIRST 2008. All Rights Reserved.							  */
/* Open Source Software - may be modified and shared by FRC teams. The code   */
/* must be accompanied by the FIRST BSD license file in $(WIND_BASE)/WPILib.  */
/*----------------------------------------------------------------------------*/

#ifndef __THUNDER_AXIS_CAMERA_H__
#define __THUNDER_AXIS_CAMERA_H__

#include <taskLib.h>
#include <vxWorks.h> 
#include <sockLib.h> 
#include <inetLib.h>

#include "WPILib.h"
#include "ThunderVideoServer.h"
#include "Vision/AxisCameraParams.h"
#include "Vision/ColorImage.h"
#include "Vision/HSLImage.h"
#include "nivision.h"
#include <set>
#include "Task.h"

/**
 * ThunderAxisCamera class.
 * This class handles everything about the Axis 206 FRC Camera.
 * It starts up 2 tasks each using a different connection to the camera:
 * - image reading task that reads images repeatedly from the camera
 * - parameter handler task in the base class that monitors for changes to
 *     parameters and updates the camera
 */
class ThunderAxisCamera: public AxisCameraParams
{
	ThunderAxisCamera(const char *cameraIP = "192.168.0.90");
public:
	virtual ~ThunderAxisCamera();
	static ThunderAxisCamera * GetInstance();
	static void DeleteInstance();

	bool IsFreshImage();
	SEM_ID GetNewImageSem();

	int GetImage(Image *imaqImage);
	int GetImage(ColorImage *image);
	HSLImage *GetImage();

	int CopyJPEG(char **destImage, int &destImageSize, int &destImageBufferSize);

	void Stop();
private:
	static int s_ImageStreamTaskFunction(ThunderAxisCamera *thisPtr);
	int ImageStreamTaskFunction();

	int ReadImagesFromCamera();
	void UpdatePublicImageFromCamera(char *imgBuffer, int imgSize);

	virtual void RestartCameraTask();

	static ThunderAxisCamera *m_instance;
	int m_cameraSocket;
	typedef std::set<SEM_ID> SemSet_t;
	SemSet_t m_newImageSemSet;
	static ThunderVideoServer *m_videoServer;

	char* m_protectedImageBuffer;
	int m_protectedImageBufferLength;
	int m_protectedImageSize;
	SEM_ID m_protectedImageSem;
	bool m_freshImage;

	Task m_imageStreamTask;
	bool m_stopImages;
	bool m_taskOver;
	
};

extern "C" {
	void ThunderAxisCameraStart();
	int ThunderAxisCameraGetImage(Image *image);
	void ThunderAxisCameraDeleteInstance();
	int ThunderAxisCameraFreshImage();

	// Mid-stream gets & writes
	void ThunderAxisCameraWriteBrightness(int brightness);
	int ThunderAxisCameraGetBrightness();
	void ThunderAxisCameraWriteWhiteBalance(AxisCameraParams::WhiteBalance_t whiteBalance);
	AxisCameraParams::WhiteBalance_t ThunderAxisCameraGetWhiteBalance();
	void ThunderAxisCameraWriteColorLevel(int colorLevel);
	int ThunderAxisCameraGetColorLevel();
	void ThunderAxisCameraWriteExposureControl(AxisCameraParams::Exposure_t exposure);
	AxisCameraParams::Exposure_t ThunderAxisCameraGetExposureControl();
	void ThunderAxisCameraWriteExposurePriority(int exposurePriority);
	int ThunderAxisCameraGetExposurePriority();
	void ThunderAxisCameraWriteMaxFPS(int maxFPS);
	int ThunderAxisCameraGetMaxFPS();

	// New-Stream gets & writes
	void ThunderAxisCameraWriteResolution(AxisCameraParams::Resolution_t resolution);
	AxisCameraParams::Resolution_t ThunderAxisCameraGetResolution();
	void ThunderAxisCameraWriteCompression(int compression);
	int ThunderAxisCameraGetCompression();
	void ThunderAxisCameraWriteRotation(AxisCameraParams::Rotation_t rotation);
	AxisCameraParams::Rotation_t ThunderAxisCameraGetRotation();
}

#endif
