#include "WPILib.h"
#include "Controls.h"
#include "ThunderDrive.h"
#include "AutoFunctions.h"
#include "IOMap.h"
#include "DashboardDataSender.h"
#include "AutoMap.h"

//Modes:
#define TELEOP_ENABLED 0
#define TELEOP_DISABLED 1
#define AUTO_ENABLED 2
#define AUTO_DISABLED 3
#define E_STOPPED 4



class ThunderBot : public SimpleRobot
{	
	ThunderDrive thunderDrive;
	Kicker kicker;
	AutoFunctions *AutoThunder;
	int automode;
	float delayTime;
	DriverStation * driveStation;
	Joystick joyLeft;
	Joystick joyRight;
	DashboardDataSender dashboardData;
	Controls *controls;
	ThunderCam *tCam;
	NewHanger *hanger;
	
public:
	
	ThunderBot(void):
		kicker(),
		driveStation(DriverStation::GetInstance()),
		joyLeft(DS_JOYNUM_LEFT), joyRight(DS_JOYNUM_RIGHT),
		dashboardData(&joyLeft, &joyRight)
	{
		GetWatchdog().SetExpiration(0.3);
		hanger = new NewHanger();
		automode = 0;
        tCam = new ThunderCam(&dashboardData);
		AutoThunder = new AutoFunctions(&kicker, &thunderDrive, tCam);
        controls = new Controls(&joyLeft, &joyRight, &thunderDrive, &kicker, tCam, hanger);
	}
	~ThunderBot()
	{
	    delete controls;
        delete AutoThunder;
	    delete tCam;
	    delete hanger;
	}

	void Autonomous(void)
	{
        GetWatchdog().SetEnabled(false);
        controls->checkForEnhancedIO();
        controls->checkDisables();

        //THIS MUST BE RUN BEFORE PROCESSKICKER RUNS FOR THE FIRST TIME.
		kicker.SetPreMatch();
		
		//sets the delay time
		delayTime = controls->getAutoDelay();
		AutoThunder->setDelay(delayTime);
		
		// set info for the balls
		AutoThunder->setBallOne(0, 0, 0, 0);
		AutoThunder->setBallTwo(0, 0, 0, 0);
		AutoThunder->setBallThree(0, 0, 0, 0);
		AutoThunder->resetState();

		// For testing
		//automode = 1;
		if (!controls->isKickerWorking())
		{
			AutoThunder->setKickerBroken(true);
		}
		else
		{
			AutoThunder->setKickerBroken(false);
		}
		switch (automode) {
			case 0:
			default:
				thunderDrive.Stop();
				AutoThunder->setModeParameters(AUTO_DFT_PWR_LVL, AUTO_DFT_DIST_BKWD, AUTO_DFT_ZONE, AUTO_DFT_CAM_MODE);
				automode = 0;// in case we hit the default case
				break;
			case 1:	//Near zone: dead reckoning
				AutoThunder->setModeParameters(AUTO_NEAR_PWR_LVL, AUTO_NEAR_DIST_BKWD, AUTO_NEAR_ZONE, AUTO_NEAR_DR_CAM_MODE);
				AutoThunder->setBallOne(AUTO_NEAR_DR_B1_ANGLE_BALL, AUTO_NEAR_DR_B1_DIST, AUTO_NEAR_DR_B1_ANGLE_GOAL, AUTO_NEAR_DR_B1_DIST_GOAL);
				break;
			/*case 2: //Near zone: camera guided
				AutoThunder->setModeParameters(AUTO_NEAR_PWR_LVL, AUTO_NEAR_DIST_BKWD, AUTO_NEAR_ZONE, AUTO_NEAR_CG_CAM_MODE);
				AutoThunder->setBallOne(AUTO_NEAR_CG_B1_ANGLE_BALL, AUTO_NEAR_CG_B1_DIST, AUTO_NEAR_CG_B1_ANGLE_GOAL, AUTO_NEAR_CG_B1_DIST_GOAL);
				break;
			case 3: //Near zone: camera see balls: not knowing if we are using yet
				AutoThunder->setModeParameters(AUTO_NEAR_PWR_LVL, AUTO_NEAR_DIST_BKWD, AUTO_NEAR_ZONE, AUTO_NEAR_CG_CAM_MODE);
				AutoThunder->setBallOne(AUTO_NEAR_CSB_B1_ANGLE_BALL, AUTO_NEAR_CSB_B1_DIST, AUTO_NEAR_CSB_B1_ANGLE_GOAL, AUTO_NEAR_CSB_B1_DIST_GOAL);
				break;*/
				
				
				
			case 2: //4: //Middle zone: dead reckoning
				AutoThunder->setModeParameters(AUTO_MID_PWR_LVL, AUTO_MID_DIST_BKWD, AUTO_MID_ZONE, AUTO_MID_DR_CAM_MODE);
				AutoThunder->setBallOne(AUTO_MID_DR_B1_ANGLE_BALL, AUTO_MID_DR_B1_DIST, AUTO_MID_DR_B1_ANGLE_GOAL, AUTO_MID_DR_B1_DIST_GOAL);
				AutoThunder->setBallTwo(AUTO_MID_DR_B2_ANGLE_BALL, AUTO_MID_DR_B2_DIST, AUTO_MID_DR_B2_ANGLE_GOAL, AUTO_MID_DR_B2_DIST_GOAL);
				break;
			/*case 5: //Middle zone: camera guided
				AutoThunder->setModeParameters(AUTO_MID_PWR_LVL, AUTO_MID_DIST_BKWD, AUTO_MID_ZONE, AUTO_MID_CG_CAM_MODE);
				AutoThunder->setBallOne(AUTO_MID_CG_B1_ANGLE_BALL, AUTO_MID_CG_B1_DIST, AUTO_MID_CG_B1_ANGLE_GOAL, AUTO_MID_CG_B1_DIST_GOAL);
				AutoThunder->setBallTwo(AUTO_MID_CG_B2_ANGLE_BALL, AUTO_MID_CG_B2_DIST, AUTO_MID_CG_B2_ANGLE_GOAL, AUTO_MID_CG_B2_DIST_GOAL);
				break;*/
				
				
			case 3: //6: //Far zone: dead reckoning
				AutoThunder->setModeParameters(AUTO_FAR_PWR_LVL, AUTO_FAR_DIST_BKWD, AUTO_FAR_ZONE, AUTO_FAR_DR_CAM_MODE);
				AutoThunder->setBallOne(AUTO_FAR_DR_B1_ANGLE_BALL, AUTO_FAR_DR_B1_DIST, AUTO_FAR_DR_B1_ANGLE_GOAL, AUTO_FAR_DR_B1_DIST_GOAL);
				AutoThunder->setBallTwo(AUTO_FAR_DR_B2_ANGLE_BALL, AUTO_FAR_DR_B2_DIST, AUTO_FAR_DR_B2_ANGLE_GOAL, AUTO_FAR_DR_B2_DIST_GOAL);
				AutoThunder->setBallThree(AUTO_FAR_DR_B3_ANGLE_BALL, AUTO_FAR_DR_B3_DIST, AUTO_FAR_DR_B3_ANGLE_GOAL, AUTO_FAR_DR_B3_DIST_GOAL);
				break;
			case 4:
				AutoThunder->setModeParameters(AUTO_BLOCK_PWR_LVL, AUTO_BLOCK_DIST_BKWD, AUTO_BLOCK_ZONE, AUTO_BLOCK_CAM_MODE);
				AutoThunder->setBallOne(AUTO_BLOCK_ANGLE_BALL, AUTO_BLOCK_DIST, AUTO_BLOCK_ANGLE_GOAL, AUTO_BLOCK_DIST_GOAL);
				break;
			// if more cases are added or removed, be sure to update
			// Disabled() where it checks the max for automode
		}

		printf("running auto mode %d\n", automode);
		while (IsAutonomous() && IsEnabled()) {
			kicker.ProcessKicker();
			if (automode != 0) {
				if(automode == 7) 
					thunderDrive.DebugAutoMode();
				else
					AutoThunder->runAuto();
			}
			dashboardData.sendIOPortData(AUTO_ENABLED, automode, delayTime);
			controls->driverstationFeedback();
			Wait(0.005);		    
		}
		// stop everything before running tele
		AutoThunder->endAuto();
	}
	void Disabled(void)
	{
		bool prevAutoPlus = false;
		bool prevAutoMinus = false;
		bool autoPlus;
		bool autoMinus;
		while(IsDisabled()) {	
			// use a maximum of 5 seconds
	        controls->checkForEnhancedIO();
			controls->runDebugging();
			delayTime = controls->getAutoDelay();
            kicker.Reset();
            
    		controls->runControls();
        	
            // if clockwise signal, then add one to automode
            // if ccw signal, then subtract one from automode
    		autoPlus = !driveStation->GetDigitalIn(DS_DIGIO_ARM_RIGHT);
            if ((autoPlus == true) && (autoPlus != prevAutoPlus)){
            	automode = automode + 1;
            }
            prevAutoPlus = autoPlus;
            autoMinus = !driveStation->GetDigitalIn(DS_DIGIO_ARM_LEFT);
            if ((autoMinus == true) && (autoMinus != prevAutoMinus)){ 
               automode = automode - 1;
            }
            prevAutoMinus = autoMinus;
            // if automode is negative, set automode to 0
            //if automode is greater than 3 set automode to 3
            if (automode < 0){
            	automode = 0;
            }
            if (automode > 3){
            	automode = 3;
            }

            int robotMode = AUTO_DISABLED; //Autonomous disabled
            if(IsOperatorControl()) robotMode = TELEOP_DISABLED; // Teleop disabled  
            dashboardData.sendIOPortData(robotMode, automode, delayTime);
            controls->driverstationFeedback();

            //Can we get camera data in disabled? Just send nothing for now
            //	dashboardData.sendVisionData(0.0, 0.0, 0.0, 0.0, NULL);

            Wait(0.005);
		}
        hanger->Reset();
	}
	
	void OperatorControl(void)
	{
		GetWatchdog().SetEnabled(true);
		kicker.SetPreMatch();
		while (IsOperatorControl() && IsEnabled())
		{
			GetWatchdog().Feed();
	        controls->checkForEnhancedIO();
			controls->runControls();  //calls the controls function
			controls->runDebugging();

			//Robot mode is Teleop Enabled (0)  
			dashboardData.sendIOPortData(TELEOP_ENABLED, automode, delayTime);
			controls->driverstationFeedback();
			kicker.ProcessKicker();
			hanger->ProcessHanger();
			Wait(0.005);				// wait for a motor update time
		}
	}
};

START_ROBOT_CLASS(ThunderBot);


