#include "WPILib.h"
#include  "stdio.h"
#include <math.h>
#include "canjaguar/CANJaguar.h"
#include "EncoderPID.h"
#include "ThunderDrive.h"
#define PID_DISTANCE_TOLERANCE 0.05f
#define PID_ANGLE_TOLERANCE 2.5f
#define MAX_AUTO_DISTANCE 132.0f
#define MIN_AUTO_OUTPUT 0.3f

// ON ROBOT
// can 7 right rear
// can 6 right front
// can 5 left rear
// can 4 left front


//IN CODE BEFORE 3/26!!!  Now fixed to match above!!!
// index 0 is 7 left front
// index 1 is 6 left rear
// index 2 is 5 right front
// index 3 is 4 Right rear
//This is where the PIDController PIDOutput goes to, but we directly read
//	the PIDController.Get() value instead to do math on it before applying to a motor
void DummyOutput::PIDWrite(float output) {}


ThunderDrive::ThunderDrive() {
	ThunderAutoMode = None;
	testModeState = 0;
	for(int i  = 0; i < 4; i++) {
		DistanceInputs[i] = new DistancePID(3, ENCODER_LEFT_FRONT_A+i);
		DummyMotors[i] = new DummyOutput();
		DistanceControllers[i] = new PIDController(0.9, 0.15, 0.0, DistanceInputs[i], DummyMotors[i]);
		Motors[i] = new DRIVESPEEDCONTROLLERS(SPEEDCONTROLLER_PORT_NUMBER);
	}
	ThunderGyro = new Gyro(GYRO);
	ThunderGyro->Reset();
	DummySteering = new DummyOutput();
	AngleController = new PIDController(0.1, 0.8, 0.0, ThunderGyro, DummySteering);
	AngleController->SetInputRange(-180, 180);
	AngleController->SetOutputRange(-180, 180);
}

ThunderDrive::~ThunderDrive() {
	for(int i  = 0; i < 4; i++) {
		delete DummyMotors[i];
		delete DistanceInputs[i];
		delete DistanceControllers[i];
		delete Motors[i];
	}
	delete ThunderGyro;
	delete DummySteering;
	delete AngleController;
}
//sets the power of the left and right motors, to be used in teleoperated mode
void ThunderDrive::SetMotorPower(float left, float right) {
	//printf("motors to l:%f, r:%f\n", left, -right);
	//These motors are backwards
	Motors[0]->Set(-left);//LF
	Motors[1]->Set(-left);//LR
	// And normal
	Motors[2]->Set(right);//RF
	Motors[3]->Set(right);//RR
}
//sets the power of each motor on the drive train, to be used in autonomous only
void ThunderDrive::SetAllMotorPower(float leftFront, float leftRear, float rightFront, float rightRear) {
	// once again, motors are backwards
	Motors[0]->Set(-leftFront);
	Motors[1]->Set(-leftRear);
	// and normal
	Motors[2]->Set(rightFront);
	Motors[3]->Set(rightRear);
}
//allows the robot to pivot around the front, based on the value given. positive is right, negetive is left
void ThunderDrive::AimRate(float rate) {
	//Drives motors in opposite directions; because left motor
	//is backwards, it goes in opposite direction given same input
	Motors[0]->Set(-rate);
	Motors[1]->Set(0);
	Motors[2]->Set(-rate);
	Motors[3]->Set(0);
}
float ThunderDrive::CalculateAverageDistanceOutput() {
	return (
		DistanceControllers[0]->Get() +
		DistanceControllers[1]->Get() +
		DistanceControllers[2]->Get() +
		DistanceControllers[3]->Get() 
	)/4;
}
//call to set settings, subsequent calls will overwrite settings
void ThunderDrive::DriveDistance(float distance, float maxSpeed) {	
	ThunderAutoMode = Drive;
	maxAutoSpeed = maxSpeed;
	// might be needed to keep autonomous from running in reverse
	//distance = -distance;
	backingUp = (distance < 0.0);
	AngleController->Reset();
	AngleController->SetSetpoint(ThunderGyro->GetAngle());
	AngleController->Enable();
	for(int i = 0; i < 4; i++) {
		DistanceInputs[i]->Reset();
		DistanceControllers[i]->Reset();
		DistanceControllers[i]->SetSetpoint(fabs(distance/MAX_AUTO_DISTANCE));///(54*12));
		DistanceControllers[i]->SetInputRange(-1, 1);
		DistanceControllers[i]->SetOutputRange(MIN_AUTO_OUTPUT, maxSpeed);
		DistanceControllers[i]->Enable();
		printf("Setpoint is %f Err: %f\n", DistanceControllers[i]->GetSetpoint(), DistanceControllers[i]->GetError());
	}
}
//call to set settings to aim to the angle given, not aim that degrees away
void ThunderDrive::AimAngle(float angle) {
	ThunderAutoMode = Angle;
	AngleController->Reset();
	AngleController->SetSetpoint(ThunderGyro->GetAngle() + angle);
	printf("Setting AngleController to %f\n", (double)(ThunderGyro->GetAngle() + angle));
	AngleController->SetInputRange(-180, 180);
	AngleController->SetOutputRange(-180, 180);
	AngleController->Enable();
}

//call to see if the robot has driven the correct distance 
bool ThunderDrive::CheckDistanceDone() {	//lets make this distancedone, or angle done
	float total = 0;
	for(int i  = 0 ; i < 4; i++) {
		total += DistanceInputs[i]->PIDGet();
	}
	total /= 4;
	double d = fabs(DistanceControllers[0]->GetSetpoint() -
					total);
	return d <  PID_DISTANCE_TOLERANCE;
}
//call to see if the robot has navigated to the correct angle
bool ThunderDrive::CheckAngleDone() {
	double d = fabs(AngleController->GetSetpoint() - 
					ThunderGyro->GetAngle());
	if (d < PID_ANGLE_TOLERANCE) {
		return true;
	} else 
		return false;
}
//call to check if the last called auto mode has completed
bool ThunderDrive::AutoMotionDone() {
	switch(ThunderAutoMode) {
		case Drive:
			return CheckDistanceDone();
			break;
		case Angle:
			return CheckAngleDone();
			break;
		default:
			return true;
			break;
	}
}
//call to stop all pids, and stop all motors
void ThunderDrive::Stop() {
	SetMotorPower(0, 0);
	for(int i = 0; i < 4; i++) {
		DistanceControllers[i]->SetSetpoint(0.0);
		DistanceControllers[i]->Disable();
		DistanceControllers[i]->Reset();
	}
	AngleController->Disable();
	AngleController->Reset();
	ThunderAutoMode = None;
}
//public function to read the gyro
float ThunderDrive::getRobotAngle(){
	return ThunderGyro->GetAngle();
}

//call at the top of every autonomous itteration
void ThunderDrive::Process() {
	//create the four values to pass to the motors after calculation
	float leftFrontPower = 0;
	float rightFrontPower = 0;
	float leftRearPower = 0;
	float rightRearPower = 0;
	//branch if driving
	if(ThunderAutoMode == Drive) {
		//calculate the distance error by averaging the distances
		float averageDistanceOutput = CalculateAverageDistanceOutput();
		//calculate the angle error, and calculate a bias based on it	
		float angleError = AngleController->GetError();
		int biasSide;
		//1 = right side biased, -1 = left side biased
		if(angleError < 1 && angleError > -1) {
			biasSide = 0;
		}
		else {
			if(angleError > 0) biasSide = -1;
			if(angleError < 0) biasSide = 1;
		}
		//at 120 degrees off, we cause the motors to reverse to turn
		float bias = (angleError/30)*maxAutoSpeed;
		//branch based on which side to bias, bias correct powers
		leftFrontPower =  averageDistanceOutput;
		rightFrontPower = averageDistanceOutput;
		if(backingUp) {
			leftFrontPower *= -1;
			rightFrontPower *= -1;
		}
		if(biasSide == -1) {
			rightFrontPower -= fabs(bias);
		}
		if(biasSide == 1) {
			leftFrontPower -= fabs(bias);
		}
		leftRearPower = leftFrontPower;
		rightRearPower = rightFrontPower;
	}
	//branch if aiming
	if(ThunderAutoMode == Angle) {
		//calcuate the power needed from the angle controllers output
		float angleError = AngleController->Get() * 1.3;
		leftFrontPower = angleError/360.0;
		rightFrontPower = -angleError/360.0;
		leftRearPower = 0;
		rightRearPower = 0;
	}
	//branch if doing nothing, i.e. code failure or end of autonomous
	if(ThunderAutoMode != Drive && ThunderAutoMode != Angle) {
		for(int i = 0; i < 4; i++) {
			DistanceControllers[i]->Disable();
			AngleController->Disable();
		}
	}
	
	//set the motor powers from calculations
	SetAllMotorPower(-leftFrontPower * 0.95,-leftRearPower * 0.95,-rightFrontPower,-rightRearPower);
	if(AutoMotionDone()) Stop();
}

void ThunderDrive::resetGyro() {
	ThunderGyro->Reset();
}
	
void ThunderDrive::DebugAutoMode() {
	Process();
	switch(testModeState) {
	case 0:
		DriveDistance(11*12, 0.35);
		testModeState = 1;
		break;
	case 1:
		if(AutoMotionDone()) {
			Stop();
			testModeState = 2;
		}
	default:
		Stop();
		break;
	}
}
double ThunderDrive::DistGet(int num) {
	return DistanceInputs[num]->DistGet();
}

void ThunderDrive::KeepOnBump() {
	Motors[0]->Set(-0.2);
	Motors[1]->Set( 0.4);
	Motors[2]->Set( 0.2);
	Motors[3]->Set(-0.4);
}
