#include "WPILib.h"	
#include "IOMap.h"	//includes nessecary information for input from joysticks and 
#include "DSMap.h"	//output to motors

#define THUNDERBOT2009 0
#if THUNDERBOT2009
#define DRIVESPEEDCONTROLLERS Victor
#define SPEEDCONTROLLER_PORT_NUMBER 1+i
#else
#include "canjaguar/CANJaguar.h"
#define DRIVESPEEDCONTROLLERS CANJaguar
#define SPEEDCONTROLLER_PORT_NUMBER 4+i
#endif	
#include "EncoderPID.h"
#ifndef THUNDER_DRIVE_H
#define THUNDER_DRIVE_H
 
//this is a dummy motor class, so that we can read the PID's without the motors reacting before the algorithm
class DummyOutput: public PIDOutput {
	public:
		virtual void PIDWrite(float output);
};

class ThunderDrive {
public:
	
	
	//Class Maintenance:
		ThunderDrive();
		~ThunderDrive();
		
		
	//General Use Functions:
		//stops all motors, disables and resets all PID's, and autoStarted is true
		void Stop();
		// resets the gyro
		// current robot position is considered 0 degrees
		void resetGyro();		
		// uses the gyro to get the current robot angle in degrees
		// left is negative, right is positive
		// 0 is starting angle
		// range is -180:180
		float getRobotAngle();
		
		
	//Teleoperated Functions:
		void returnCounts();
		//Left is the power of the left motors, right is the power of the right motors
		//Range is -1 to 1, -1 is full reverse and 1 is full forward
		//there is no need to negate the right, that is done automatically
		void SetMotorPower(float left, float right);
	
		//Turns just the front motors at the float power you pass it
		//Range is -1 to 1, -1 full left and 1 full right
		void AimRate(float rate);
		
		//causes motors to fight against one another to keep
		//the robot from moving
		void KeepOnBump();
	
		
	//Autonomous Functions:
		//Turns the robot on a dime the angle you pass it FROM IT'S CURRENT ANGLE
		//Range is -180 to 180, negative numbers being left and positive being right
		//Turns to the gyro angle (Gyro relative, not robot relative)
		//Returns whether it is at that angle (true) or not (false)
		//If it is done it stops the motors
		void AimAngle(float angle);
	
		//Drives the distance FROM YOUR CURRENT POSITION 
		//(+ or -) in inches you pass it, straight
		//Stops the motors when it's done
		//Max speed range goes from 0 to 1
		void DriveDistance(float distance, float maxSpeed = 1.0);
		
		//call to check if last used auto function has completed
		bool AutoMotionDone();
		
		void DebugAutoMode();
		// Call every main program loop iteration to process
		// any ongoing drive operations.
		void Process();

		double DistGet(int num); //Gets encoder value for debugging purposes
		
private:
	
	
	//PID Sources:
		//will be used to find output for motors, and that data manipulated to allow for straight driving
		//Used By: AngleController
		Gyro * ThunderGyro;
		
		//the inputs for distance controllers, based of distance retreived from counter clicks
		//Used By: DistanceControllers[i]
		DistancePID * DistanceInputs[4]; 
		
	//PID Outputs:
		//the dummy motor for PID's to write to, so the output can be manipulated
		//Used By: DistanceControllers[i]
		DummyOutput * DummyMotors[4];
	
		//the dummy output for the steering pid to write to
		//Used By: AngleController
		DummyOutput * DummySteering;
		
		//array of four motors for driving, will be changed to CANJaguar for thunderFoot
		//DRIVESPEEDCONTROLLERS is defined as CANJaguar or Victor in the .h
		DRIVESPEEDCONTROLLERS * Motors[4];
		
		
	//PID Controllers:
		//pid controller for use with gyro
		//Source: ThunderGyro
		//Output: DummySteering
		PIDController * AngleController;	
		
		//PIDControllers (0-3)  for use with DriveDistance
		//Source: DistanceInputs[i]
		//Output: DummyMotors[i]
		PIDController * DistanceControllers[4];
		
		
	//Auto Calls:
		//whether or not the last called drive distance is driving forward
		bool DrivingForward;
		int testModeState;
		bool backingUp; 
		//the last called autonomous mode
		typedef enum {Drive, Angle, None} AutoMode;
		AutoMode ThunderAutoMode;
		
		//the max speed the distance PID can use
		float maxAutoSpeed;
		
		//average all distance PID outputs
		float CalculateAverageDistanceOutput();
		
		//check if the distance PID's have an error less than the distance tolerance
		bool CheckDistanceDone();
		
		//check if the steering PID has an error less than the angle tolerance
		bool CheckAngleDone();
		
		// lets you control the power of all the different motors, not just the left ones and right ones combined
		// range is -1 to 1, full reverse is -1, full forward is 1, 0 is stop
		// no need to negate the right, that is done already
		// mainly for autonomous so that when it turns it can turn around the front wheels like AimRate
		void SetAllMotorPower(float leftFront, float leftRear, float rightFront, float rightRear);
};



#endif

