#include "ThunderGlow.h"
#define ALLIANCE_OVERRIDE 1
ThunderGlow::ThunderGlow() {
	driverStation = DriverStation::GetInstance();
	time = 0;
	pulsingStep = ThunderGlow::getStartTime;
	lastPowerChange = ThunderGlow::Off;
	glow = new Relay(RELAY_UNDERGLOW, Relay::kForwardOnly);
}
ThunderGlow::~ThunderGlow() {
	delete glow;
}

void ThunderGlow::controlGlow(int direction) {
	if (direction == 1)  {
		glow->Set(Relay::kOn);
		lastPowerChange = ThunderGlow::On;
	}
	else {
		glow->Set(Relay::kOff);
		lastPowerChange = ThunderGlow::Off;
	}
}

void ThunderGlow::pulseGlow(double rate) {
	switch (pulsingStep) {
		case ThunderGlow::getStartTime:
			time = GetTime();
			if (lastPowerChange == ThunderGlow::On) {
				controlGlow(0);
				//printf("Turning Off Light\n");
				pulsingStep = ThunderGlow::checkPulse;
			}
			else {
				controlGlow(1);
				//printf("Turning On Lights\n");
				pulsingStep = ThunderGlow::checkPulse;
			}
			break;
		case ThunderGlow::checkPulse:
			if (GetTime() > (time + rate)) {
				pulsingStep = ThunderGlow::getStartTime;
				//printf("Time is Completed %f %f %f\n", (double)GetTime(), (double)time, (double)rate);
			}
			else {
				//printf("Not at Time Yet %f %f %f\n", (double)GetTime(), (double)time, (double)rate);
			}
			break;
	}
}

void ThunderGlow::Process(bool shooterNotReady, bool armLocked, bool ballAvailable) {
	// always OFF if not Red alliance
#if !ALLIANCE_OVERRIDE
	if (driverStation->GetAlliance() != DriverStation::kRed) {
		controlGlow(0);
		pulsingStep = ThunderGlow::getStartTime; //reset pulsing
		return;
	}
#endif

	// always flash when the arm is locked - more important then ball sensor
	if (armLocked) {
		printf("arm locked\n");
		pulseGlow(0.2);
		return;
	}
	// always ON when there is a ball
	if (ballAvailable) {
		controlGlow(1);
		pulsingStep = ThunderGlow::getStartTime; //reset pulsing
		return;
	}
	if (shooterNotReady) {
		pulseGlow(0.2);
	}
	else {
		// done flashing, so make sure it is OFF
		controlGlow(0);
		pulsingStep = ThunderGlow::getStartTime; //reset pulsing
	}
}
